/*
 * titraxDB.c
 *
 * Titrax database subroutines
 *
 * $Id: titraxDB.c,v 1.3 1997/11/23 15:30:38 ga58 Exp ga58 $
 *
 * $Log: titraxDB.c,v $
 * Revision 1.3  1997/11/23 15:30:38  ga58
 * - fixed a off-by-one bug in ReadNote
 *
 * Revision 1.2  1997/11/15 17:59:29  ga58
 * *** empty log message ***
 *
 * Revision 1.1  1997/11/14 17:51:04  ga58
 * Initial revision
 *
 */

#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <UI/UIGlobals.h>

#include "titraxAll.h"

static DmOpenRef TitraxDB;

#define minNoteLen 64

/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */

void
OpenDatabase(void)
{
  UInt mode;

  mode = dmModeReadWrite;
  TitraxDB = DmOpenDatabaseByTypeCreator(titraxDBType, titraxAppType, mode);
  ErrFatalDisplayIf(!TitraxDB, "Could not create Titrax database.");
}

void
CreateDatabase(void)
{
  Err err;
  LocalID dbID;

  /* check for a existing database */
  dbID = DmFindDatabase(0, titraxDBName);
  if (dbID != 0)
	 DmDeleteDatabase(0, dbID);

  err = DmCreateDatabase(0, titraxDBName, titraxAppType, titraxDBType, false);
  ErrFatalDisplayIf(err, "Could not create database.");
}

void
CloseDatabase(void)
{
  DmCloseDatabase(TitraxDB);
}

Int
CreateRecord(Int idx, Int size)
{
  Int i;
  Err error;
  VoidPtr recP;
  VoidHand recH;

  i = idx;

  recH = DmNewRecord(TitraxDB, &i, size);
  ErrFatalDisplayIf(recH == 0, "Cannot create new record handle.");
  recP = MemHandleLock(recH);
  ErrFatalDisplayIf(recP == 0, "Cannot lock new record handle.");

  error = DmSet(recP, 0, size, 0);
  ErrFatalDisplayIf(error, "Could not create record.");

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, i, true);

  return i;
}

void
CreateRecords(void)
{
  Int i;
  Int index = 0;

  for(i = 0; i < MAXSLOT; i++) {
	 SetRunningRecIdx(i, CreateRecord(index, sizeof(Boolean)));
	 index++;
  }

  for(i = 0; i < MAXSLOT; i++) {
	 SetElapsedRecIdx(i, CreateRecord(index, sizeof(ULong)));
	 index++;
  }

  for(i = 0; i < MAXSLOT; i++) {
	 SetStartedAtRecIdx(i, CreateRecord(index, sizeof(ULong)));
	 index++;
  }

  for(i = 0; i < MAXSLOT; i++) {
	 SetTextRecIdx(i, CreateRecord(index, MAXTEXT+1));
	 index++;
  }

  for(i = 0; i < MAXSLOT; i++) {
	 SetNoteRecIdx(i, CreateRecord(index, minNoteLen));
	 index++;
  }
}

void
WriteRecord(Int idx, VoidPtr p, Int size, Int cSize)
{
  VoidPtr recP;
  VoidHand recH;

  recH = DmGetRecord(TitraxDB, idx);
  ErrFatalDisplayIf(recH == 0, "Cannot get record handle.");
  recP = MemHandleLock(recH);
  ErrFatalDisplayIf(recP == 0, "Cannot lock record handle.");

  DmSet(recP, 0, cSize, 0);
  DmWrite(recP, 0, p, size);

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, idx, true);
}

void 
ReadRecord(Int idx, VoidPtr p, Int size)
{
  VoidPtr recP;
  VoidHand recH;

  recH = DmQueryRecord(TitraxDB, idx);
  recP = MemHandleLock(recH);

  MemMove(p, recP, size);

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, idx, false);
}

Handle
GetRecordHandle(Int idx)
{
  return (Handle)DmGetRecord(TitraxDB, idx);
}

void
ReleaseRecordHandle(Int idx, Boolean dirty)
{
  DmReleaseRecord(TitraxDB, idx, dirty);
}

Boolean
IsRunning(Int no)
{
  Boolean val;
  ReadRecord(RunningRecIdx(no), &val, sizeof(val));
  return val;
}

void
SetRunning(Int no, Boolean val)
{
  WriteRecord(RunningRecIdx(no), &val, sizeof(val), sizeof(val));
}

ULong
GetElapsed(Int no)
{
  ULong val;
  ReadRecord(ElapsedRecIdx(no), &val, sizeof(val));
  return val;
}

void
SetElapsed(Int no, ULong val)
{
  WriteRecord(ElapsedRecIdx(no), &val, sizeof(val), sizeof(val));
}

ULong
GetStartedAt(Int no)
{
  ULong val;
  ReadRecord(StartedAtRecIdx(no), &val, sizeof(val));
  return val;
}
  
void
SetStartedAt(Int no, ULong val)
{
  WriteRecord(StartedAtRecIdx(no), &val, sizeof(val), sizeof(val));
}

void
GetText(Int no, CharPtr charP)
{
  ReadRecord(TextRecIdx(no), charP, MAXTEXT+1);
}

void
SetText(Int no, CharPtr charP)
{
  WriteRecord(TextRecIdx(no), charP, StrLen(charP), MAXTEXT+1);
}

Boolean
TextIsEmpty(Int no)
{
  Char t[MAXTEXT+1];

  ReadRecord(TextRecIdx(no), t, MAXTEXT+1);

  return (StrCompare(t, "") == 0);
}

/*
 * note subroutines. handled different because the notes should have
 * a dynamic size
 */
void 
WriteNoteRecord(Int idx, CharPtr charP, Int len)
{
  ULong newLen;
  VoidPtr recP;
  VoidHand recH;

  newLen = (len > minNoteLen) ? len : minNoteLen;

  recH = DmResizeRecord(TitraxDB, idx, newLen);
  ErrFatalDisplayIf(recH == 0, "Note record resize failed.");
  recP = MemHandleLock(recH);
  ErrFatalDisplayIf(recP == 0, "Note record lock failed.");

  DmSet(recP, 0, newLen, 0);
  DmStrCopy(recP, 0, charP);

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, idx, true);
}

VoidHand
ReadNoteRecord(Int idx)
{
  Int len;
  VoidPtr recP, strP;
  VoidHand recH, strH;

  recH = DmQueryRecord(TitraxDB, idx);
  recP = MemHandleLock(recH);

  len = StrLen(recP) + 1;

  strH = MemHandleNew(len);
  strP = MemHandleLock(strH);

  MemSet(strP, len, 0);
  MemMove(strP, recP, StrLen(recP));
  MemHandleUnlock(strH);

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, idx, false);

  return strH;
}

Int
NoteStrLen(Int idx)
{
  Int len;
  VoidPtr recP;
  VoidHand recH;

  recH = DmQueryRecord(TitraxDB, idx);
  recP = MemHandleLock(recH);

  len = StrLen(recP);

  MemHandleUnlock(recH);
  DmReleaseRecord(TitraxDB, idx, false);

  return len;
}

void
SetNote(Int no, CharPtr charP)
{
  WriteNoteRecord(NoteRecIdx(no), charP, StrLen(charP) + 1);
}
  
VoidHand
GetNote(Int no)
{
  return ReadNoteRecord(NoteRecIdx(no));
}

Boolean
NoteIsEmpty(Int no)
{
  return (NoteStrLen(NoteRecIdx(no)) == 0);
}

void
ClearNote(Int no)
{
  Char str[2];

  StrCopy(str, "");
  WriteNoteRecord(NoteRecIdx(no), str, StrLen(str) + 1);
}

Int 
NoteLen(Int no)
{
  return NoteStrLen(NoteRecIdx(no));
}
